/*
 *
 * ====================================================================
 *
 * The P6Spy Software License, Version 1.1
 *
 * This license is derived and fully compatible with the Apache Software
 * license, see http://www.apache.org/LICENSE.txt
 *
 * Copyright (c) 2001-2002 Andy Martin, Ph.D. and Jeff Goke
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 * any, must include the following acknowlegement:
 * "The original concept and code base for P6Spy was conceived
 * and developed by Andy Martin, Ph.D. who generously contribued
 * the first complete release to the public under this license.
 * This product was due to the pioneering work of Andy
 * that began in December of 1995 developing applications that could
 * seamlessly be deployed with minimal effort but with dramatic results.
 * This code is maintained and extended by Jeff Goke and with the ideas
 * and contributions of other P6Spy contributors.
 * (http://www.p6spy.com)"
 * Alternately, this acknowlegement may appear in the software itself,
 * if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "P6Spy", "Jeff Goke", and "Andy Martin" must not be used
 * to endorse or promote products derived from this software without
 * prior written permission. For written permission, please contact
 * license@p6spy.com.
 *
 * 5. Products derived from this software may not be called "P6Spy"
 * nor may "P6Spy" appear in their names without prior written
 * permission of Jeff Goke and Andy Martin.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/**
 * Description: Implements the StringMatcher with simple substring matching.
 *              With pre-jdk1.4 jvms, this should be the default matching used.
 *
 * $Author: jeffgoke $
 * $Revision: 1.1 $
 * $Date: 2002/05/24 07:32:01 $
 *
 * $Id: GnuRegexMatcher.java,v 1.1 2002/05/24 07:32:01 jeffgoke Exp $
 * $Log: GnuRegexMatcher.java,v $
 * Revision 1.1  2002/05/24 07:32:01  jeffgoke
 * version 1 rewrite
 *
 * Revision 1.1  2002/04/22 02:27:04  jeffgoke
 * Simon Sadedin added timing information.  Added Junit tests.
 *
 */
package com.p6spy.engine.common;

import gnu.regexp.RE;
import gnu.regexp.REException;

import java.util.Hashtable;

public class GnuRegexMatcher implements StringMatcher {

   private Hashtable expressionCache = new Hashtable(10);

  /**
   * Returns true if the given expression is found within the value.
   *
   * @param expression - the expression to find
   * @param value - the value to look for the expression in
   * @return - true if the expression is found, false otherwise
   */
  public boolean match(String expression, String value) throws MatchException {

      try {
          // ssadedin: MT safety - I think the risk of MT collision
          // here is ok because the worst case is that a regex gets compiled twice
          if(!expressionCache.contains(expression)) {
              expressionCache.put(expression, new RE(expression));
          }

          RE regex = (RE)expressionCache.get(expression);

          return regex.getMatch(value) != null;
      } catch (REException e) {
          throw new MatchException("Unable to match expression: [" + expression + "]", e);
      }
  }
}
